@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2008, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -46,6 +46,11 @@
 #define RDF_PREFIX TRACKER_RDF_PREFIX
 #define RDFS_PREFIX TRACKER_RDFS_PREFIX
 #define TRACKER_PREFIX TRACKER_TRACKER_PREFIX
+#define NAO_PREFIX TRACKER_NAO_PREFIX
+#define NAO_LAST_MODIFIED NAO_PREFIX "lastModified"
+#define RDF_PREFIX TRACKER_RDF_PREFIX
+#define RDF_PROPERTY RDF_PREFIX "Property"
+#define RDF_TYPE RDF_PREFIX "type"
 
 typedef struct _TrackerDataUpdateBuffer TrackerDataUpdateBuffer;
 typedef struct _TrackerDataUpdateBufferResource TrackerDataUpdateBufferResource;
@@ -124,6 +129,14 @@ struct _TrackerCommitDelegate {
 	gpointer user_data;
 };
 
+typedef struct {
+	gchar *graph;
+	gchar *subject;
+	gchar *predicate;
+	gchar *object;
+	gboolean is_uri;
+} QueuedStatement;
+
 static gboolean in_transaction = FALSE;
 static gboolean in_journal_replay = FALSE;
 static TrackerDataUpdateBuffer update_buffer;
@@ -136,6 +149,8 @@ static GPtrArray *insert_callbacks = NULL;
 static GPtrArray *delete_callbacks = NULL;
 static GPtrArray *commit_callbacks = NULL;
 static GPtrArray *rollback_callbacks = NULL;
+static gint max_service_id = 0;
+static gint max_modseq = 0;
 
 static gint ensure_resource_id (const gchar *uri, gboolean    *create);
 
@@ -155,6 +170,32 @@ tracker_data_add_commit_statement_callback (TrackerCommitCallback    callback,
 	g_ptr_array_add (commit_callbacks, delegate);
 }
 
+void
+tracker_data_remove_commit_statement_callback (TrackerCommitCallback callback,
+                                               gpointer              user_data)
+{
+	TrackerCommitDelegate *delegate;
+	guint i;
+	gboolean found = FALSE;
+
+	if (!commit_callbacks) {
+		return;
+	}
+
+	for (i = 0; i < commit_callbacks->len; i++) {
+		delegate = g_ptr_array_index (commit_callbacks, i);
+		if (delegate->callback == callback && delegate->user_data == user_data) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (found) {
+		g_free (delegate);
+		g_ptr_array_remove_index (commit_callbacks, i);
+	}
+}
+
 void
 tracker_data_add_rollback_statement_callback (TrackerCommitCallback    callback,
                                               gpointer                 user_data)
@@ -171,6 +212,33 @@ tracker_data_add_rollback_statement_callback (TrackerCommitCallback    callback,
 	g_ptr_array_add (rollback_callbacks, delegate);
 }
 
+
+void
+tracker_data_remove_rollback_statement_callback (TrackerCommitCallback callback,
+                                                 gpointer              user_data)
+{
+	TrackerCommitDelegate *delegate;
+	guint i;
+	gboolean found = FALSE;
+
+	if (!rollback_callbacks) {
+		return;
+	}
+
+	for (i = 0; i < rollback_callbacks->len; i++) {
+		delegate = g_ptr_array_index (rollback_callbacks, i);
+		if (delegate->callback == callback && delegate->user_data == user_data) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (found) {
+		g_free (delegate);
+		g_ptr_array_remove_index (rollback_callbacks, i);
+	}
+}
+
 void
 tracker_data_add_insert_statement_callback (TrackerStatementCallback callback,
                                             gpointer                 user_data)
@@ -187,6 +255,32 @@ tracker_data_add_insert_statement_callback (TrackerStatementCallback callback,
 	g_ptr_array_add (insert_callbacks, delegate);
 }
 
+void
+tracker_data_remove_insert_statement_callback (TrackerStatementCallback callback,
+                                               gpointer                 user_data)
+{
+	TrackerStatementDelegate *delegate;
+	guint i;
+	gboolean found = FALSE;
+
+	if (!insert_callbacks) {
+		return;
+	}
+
+	for (i = 0; i < insert_callbacks->len; i++) {
+		delegate = g_ptr_array_index (insert_callbacks, i);
+		if (delegate->callback == callback && delegate->user_data == user_data) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (found) {
+		g_free (delegate);
+		g_ptr_array_remove_index (insert_callbacks, i);
+	}
+}
+
 void
 tracker_data_add_delete_statement_callback (TrackerStatementCallback callback,
                                             gpointer                 user_data)
@@ -203,6 +297,32 @@ tracker_data_add_delete_statement_callback (TrackerStatementCallback callback,
 	g_ptr_array_add (delete_callbacks, delegate);
 }
 
+void
+tracker_data_remove_delete_statement_callback (TrackerStatementCallback callback,
+                                               gpointer                 user_data)
+{
+	TrackerStatementDelegate *delegate;
+	guint i;
+	gboolean found = FALSE;
+
+	if (!delete_callbacks) {
+		return;
+	}
+
+	for (i = 0; i < delete_callbacks->len; i++) {
+		delegate = g_ptr_array_index (delete_callbacks, i);
+		if (delegate->callback == callback && delegate->user_data == user_data) {
+			found = TRUE;
+			break;
+		}
+	}
+
+	if (found) {
+		g_free (delegate);
+		g_ptr_array_remove_index (delete_callbacks, i);
+	}
+}
+
 GQuark tracker_data_error_quark (void) {
 	return g_quark_from_static_string ("tracker_data_error-quark");
 }
@@ -214,10 +334,8 @@ tracker_data_update_get_new_service_id (void)
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
 
-	static gint         max = 0;
-
-	if (G_LIKELY (max != 0)) {
-		return ++max;
+	if (G_LIKELY (max_service_id != 0)) {
+		return ++max_service_id;
 	}
 
 	iface = tracker_db_manager_get_db_interface ();
@@ -229,11 +347,18 @@ tracker_data_update_get_new_service_id (void)
 
 	if (cursor) {
 		tracker_db_cursor_iter_next (cursor);
-		max = MAX (tracker_db_cursor_get_int (cursor, 0), max);
+		max_service_id = MAX (tracker_db_cursor_get_int (cursor, 0), max_service_id);
 		g_object_unref (cursor);
 	}
 
-	return ++max;
+	return ++max_service_id;
+}
+
+void
+tracker_data_update_shutdown (void)
+{
+	max_service_id = 0;
+	max_modseq = 0;
 }
 
 static gint
@@ -242,10 +367,9 @@ tracker_data_update_get_next_modseq (void)
 	TrackerDBCursor    *cursor;
 	TrackerDBInterface *temp_iface;
 	TrackerDBStatement *stmt;
-	static gint         max = 0;
 
-	if (G_LIKELY (max != 0)) {
-		return ++max;
+	if (G_LIKELY (max_modseq != 0)) {
+		return ++max_modseq;
 	}
 
 	temp_iface = tracker_db_manager_get_db_interface ();
@@ -257,11 +381,11 @@ tracker_data_update_get_next_modseq (void)
 
 	if (cursor) {
 		tracker_db_cursor_iter_next (cursor);
-		max = MAX (tracker_db_cursor_get_int (cursor, 0), max);
+		max_modseq = MAX (tracker_db_cursor_get_int (cursor, 0), max_modseq);
 		g_object_unref (cursor);
 	}
 
-	return ++max;
+	return ++max_modseq;
 }
 
 
@@ -2220,19 +2344,155 @@ tracker_data_sync (void)
 	tracker_db_journal_fsync ();
 }
 
+static void
+free_queued_statement (QueuedStatement *queued)
+{
+	g_free (queued->subject);
+	g_free (queued->predicate);
+	g_free (queued->object);
+	g_free (queued->graph);
+	g_free (queued);
+}
+
+static GList*
+queue_statement (GList *queue, 
+                 const gchar *graph,
+                 const gchar *subject,
+                 const gchar *predicate,
+                 const gchar *object,
+                 gboolean     is_uri)
+{
+	QueuedStatement *queued = g_new (QueuedStatement, 1);
+
+	queued->subject = g_strdup (subject);
+	queued->predicate = g_strdup (predicate);
+	queued->object = g_strdup (object);
+	queued->is_uri = is_uri;
+	queued->graph = graph ? g_strdup (graph) : NULL;
+
+	queue = g_list_append (queue, queued);
+
+	return queue;
+}
+
+static void
+ontology_transaction_end (GList *ontology_queue,
+                          GPtrArray *seen_classes,
+                          GPtrArray *seen_properties)
+{
+	GList *l;
+	const gchar *ontology_uri = NULL;
+
+	tracker_data_ontology_process_changes (seen_classes, seen_properties);
+
+	/* Perform ALTER-TABLE and CREATE-TABLE calls for all that are is_new */
+	tracker_data_ontology_import_into_db (TRUE);
+
+	for (l = ontology_queue; l; l = l->next) {
+		QueuedStatement *queued = ontology_queue->data;
+
+		if (g_strcmp0 (queued->predicate, RDF_TYPE) == 0) {
+			if (g_strcmp0 (queued->object, TRACKER_PREFIX "Ontology") == 0) {
+				ontology_uri = queued->subject;
+			}
+		}
+
+		/* store ontology in database */
+		tracker_data_ontology_process_statement (queued->graph,
+		                                         queued->subject, 
+		                                         queued->predicate, 
+		                                         queued->object, 
+		                                         queued->is_uri, 
+		                                         TRUE, TRUE);
+
+	}
+
+	/* Update the nao:lastModified in the database */
+	if (ontology_uri) {
+		TrackerOntology *ontology;
+		ontology = tracker_ontologies_get_ontology_by_uri (ontology_uri);
+		if (ontology) {
+			gint last_mod = 0;
+			TrackerDBInterface *iface;
+			TrackerDBStatement *stmt;
+
+			iface = tracker_db_manager_get_db_interface ();
+
+			/* We can't do better than this cast, it's stored as an int in the
+			 * db. See tracker-data-manager.c for more info. */
+			last_mod = (gint) tracker_ontology_get_last_modified (ontology);
+
+			stmt = tracker_db_interface_create_statement (iface,
+			        "UPDATE \"rdfs:Resource\" SET \"nao:lastModified\"= ? "
+			        "WHERE \"rdfs:Resource\".ID = "
+			        "(SELECT Resource.ID FROM Resource INNER JOIN \"rdfs:Resource\" "
+			        "ON \"rdfs:Resource\".ID = Resource.ID WHERE "
+			        "Resource.Uri = ?)");
+
+			tracker_db_statement_bind_int (stmt, 0, last_mod);
+			tracker_db_statement_bind_text (stmt, 1, ontology_uri);
+			tracker_db_statement_execute (stmt, NULL);
+		}
+	}
+
+	/* Reset the is_new flag for all classes and properties */
+	tracker_data_ontology_import_finished ();
+}
+
+static GList*
+ontology_statement_insert (GList       *ontology_queue,
+                           gint         graph_id,
+                           gint         subject_id,
+                           gint         predicate_id,
+                           const gchar *object,
+                           GHashTable  *classes,
+                           GHashTable  *properties,
+                           GHashTable  *id_uri_map,
+                           gboolean     is_uri,
+                           GPtrArray   *seen_classes,
+                           GPtrArray   *seen_properties)
+{
+	const gchar *graph, *subject, *predicate;
+
+	if (graph_id > 0) {
+		graph = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (graph_id));
+	} else {
+		graph = NULL;
+	}
+
+	subject = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (subject_id));
+	predicate = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (predicate_id));
+
+	/* load ontology from journal into memory, set all new's is_new to TRUE */
+	tracker_data_ontology_load_statement ("journal", subject_id, subject, predicate,
+	                                      object, NULL, TRUE, classes, properties,
+	                                      seen_classes, seen_properties);
+
+	/* Queue the statement for processing after ALTER in ontology_transaction_end */
+	ontology_queue = queue_statement (ontology_queue, graph, subject, predicate, object, is_uri);
+
+	return ontology_queue;
+}
+
 void
-tracker_data_replay_journal (GHashTable *classes,
-                             GHashTable *properties)
+tracker_data_replay_journal (GHashTable          *classes,
+                             GHashTable          *properties,
+                             GHashTable          *id_uri_map,
+                             TrackerBusyCallback  busy_callback,
+                             gpointer             busy_user_data,
+                             const gchar         *busy_status)
 {
 	GError *journal_error = NULL;
-	static TrackerProperty *rdf_type = NULL;
+	TrackerProperty *rdf_type = NULL;
 	gint last_operation_type = 0;
+	gboolean in_ontology = FALSE;
+	GList *ontology_queue = NULL;
+	GPtrArray *seen_classes = NULL;
+	GPtrArray *seen_properties = NULL;
 
 	tracker_data_begin_db_transaction_for_replay (0);
 
-	if (!rdf_type) {
-		rdf_type = tracker_ontologies_get_property_by_uri (RDF_PREFIX "type");
-	}
+	rdf_type = tracker_ontologies_get_property_by_uri (RDF_PREFIX "type");
 
 	tracker_db_journal_reader_init (NULL);
 
@@ -2253,6 +2513,11 @@ tracker_data_replay_journal (GHashTable *classes,
 
 			tracker_db_journal_reader_get_resource (&id, &uri);
 
+			if (in_ontology) {
+				g_hash_table_insert (id_uri_map, GINT_TO_POINTER (id), (gpointer) uri);
+				continue;
+			}
+
 			class = g_hash_table_lookup (classes, GINT_TO_POINTER (id));
 			if (!class)
 				property = g_hash_table_lookup (properties, GINT_TO_POINTER (id));
@@ -2270,25 +2535,63 @@ tracker_data_replay_journal (GHashTable *classes,
 			tracker_db_statement_bind_int (stmt, 0, id);
 			tracker_db_statement_bind_text (stmt, 1, uri);
 			tracker_db_statement_execute (stmt, &new_error);
+			g_object_unref (stmt);
 
 			if (new_error) {
 				g_warning ("Journal replay error: '%s'", new_error->message);
 				g_error_free (new_error);
 			}
 
+		} else if (type == TRACKER_DB_JOURNAL_START_ONTOLOGY_TRANSACTION) {
+			in_ontology = TRUE;
 		} else if (type == TRACKER_DB_JOURNAL_START_TRANSACTION) {
 			resource_time = tracker_db_journal_reader_get_time ();
 		} else if (type == TRACKER_DB_JOURNAL_END_TRANSACTION) {
-			GError *new_error = NULL;
-			tracker_data_update_buffer_might_flush (&new_error);
-			if (new_error) {
-				g_warning ("Journal replay error: '%s'", new_error->message);
-				g_clear_error (&new_error);
+			if (in_ontology) {
+				ontology_transaction_end (ontology_queue, seen_classes, seen_properties);
+				g_list_foreach (ontology_queue, (GFunc) free_queued_statement, NULL);
+				g_list_free (ontology_queue);
+				ontology_queue = NULL;
+				in_ontology = FALSE;
+				tracker_data_ontology_free_seen (seen_classes);
+				seen_classes = NULL;
+				tracker_data_ontology_free_seen (seen_properties);
+				seen_properties = NULL;
+			} else {
+				GError *new_error = NULL;
+				tracker_data_update_buffer_might_flush (&new_error);
+				if (new_error) {
+					g_warning ("Journal replay error: '%s'", new_error->message);
+					g_clear_error (&new_error);
+				}
 			}
 		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT) {
 			GError *new_error = NULL;
 			TrackerProperty *property;
 
+			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
+
+			if (in_ontology) {
+
+				if (!seen_classes)
+					seen_classes = g_ptr_array_new ();
+				if (!seen_properties)
+					seen_properties = g_ptr_array_new ();
+
+				ontology_queue = ontology_statement_insert (ontology_queue,
+				                                            graph_id,
+				                                            subject_id,
+				                                            predicate_id,
+				                                            object,
+				                                            classes,
+				                                            properties,
+				                                            id_uri_map,
+				                                            FALSE,
+				                                            seen_classes,
+				                                            seen_properties);
+				continue;
+			}
+
 			if (last_operation_type == -1) {
 				tracker_data_update_buffer_flush (&new_error);
 				if (new_error) {
@@ -2298,8 +2601,6 @@ tracker_data_replay_journal (GHashTable *classes,
 			}
 			last_operation_type = 1;
 
-			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
-
 			property = g_hash_table_lookup (properties, GINT_TO_POINTER (predicate_id));
 
 			if (property) {
@@ -2321,6 +2622,31 @@ tracker_data_replay_journal (GHashTable *classes,
 			TrackerClass *class = NULL;
 			TrackerProperty *property;
 
+			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
+
+			if (in_ontology) {
+				const gchar *object_n;
+				object_n = g_hash_table_lookup (id_uri_map, GINT_TO_POINTER (object_id));
+
+				if (!seen_classes)
+					seen_classes = g_ptr_array_new ();
+				if (!seen_properties)
+					seen_properties = g_ptr_array_new ();
+
+				ontology_queue = ontology_statement_insert (ontology_queue,
+				                                            graph_id,
+				                                            subject_id,
+				                                            predicate_id,
+				                                            object_n,
+				                                            classes,
+				                                            properties,
+				                                            id_uri_map,
+				                                            TRUE,
+				                                            seen_classes,
+				                                            seen_properties);
+				continue;
+			}
+
 			if (last_operation_type == -1) {
 				tracker_data_update_buffer_flush (&new_error);
 				if (new_error) {
@@ -2330,8 +2656,6 @@ tracker_data_replay_journal (GHashTable *classes,
 			}
 			last_operation_type = 1;
 
-			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
-
 			property = g_hash_table_lookup (properties, GINT_TO_POINTER (predicate_id));
 
 			if (property) {
@@ -2368,6 +2692,12 @@ tracker_data_replay_journal (GHashTable *classes,
 			GError *new_error = NULL;
 			TrackerProperty *property;
 
+			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
+
+			if (in_ontology) {
+				continue;
+			}
+
 			if (last_operation_type == 1) {
 				tracker_data_update_buffer_flush (&new_error);
 				if (new_error) {
@@ -2377,8 +2707,6 @@ tracker_data_replay_journal (GHashTable *classes,
 			}
 			last_operation_type = -1;
 
-			tracker_db_journal_reader_get_statement (&graph_id, &subject_id, &predicate_id, &object);
-
 			resource_buffer_switch (NULL, graph_id, NULL, subject_id);
 
 			property = g_hash_table_lookup (properties, GINT_TO_POINTER (predicate_id));
@@ -2413,6 +2741,12 @@ tracker_data_replay_journal (GHashTable *classes,
 			TrackerClass *class = NULL;
 			TrackerProperty *property;
 
+			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
+
+			if (in_ontology) {
+				continue;
+			}
+
 			if (last_operation_type == 1) {
 				tracker_data_update_buffer_flush (&new_error);
 				if (new_error) {
@@ -2422,8 +2756,6 @@ tracker_data_replay_journal (GHashTable *classes,
 			}
 			last_operation_type = -1;
 
-			tracker_db_journal_reader_get_statement_id (&graph_id, &subject_id, &predicate_id, &object_id);
-
 			property = g_hash_table_lookup (properties, GINT_TO_POINTER (predicate_id));
 
 			if (property) {
@@ -2451,8 +2783,13 @@ tracker_data_replay_journal (GHashTable *classes,
 				g_warning ("Journal replay error: 'property with ID %d doesn't exist'", predicate_id);
 			}
 		}
-	}
 
+		if (busy_callback) {
+			busy_callback (busy_status,
+			               tracker_db_journal_reader_get_progress (),
+			               busy_user_data);
+		}
+	}
 
 	if (journal_error) {
 		gsize size;
@@ -2470,5 +2807,4 @@ tracker_data_replay_journal (GHashTable *classes,
 	}
 
 	tracker_data_commit_db_transaction ();
-
 }
