@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -222,7 +222,7 @@ test_ontology_change (void)
 	gchar *ontology_dir;
 	guint i;
 	GError *error = NULL;
-	gchar *test_schemas[4] = { NULL, NULL, NULL, NULL };
+	gchar *test_schemas[5] = { NULL, NULL, NULL, NULL, NULL };
 
 	delete_db (TRUE);
 
@@ -231,7 +231,8 @@ test_ontology_change (void)
 
 	test_schemas[0] = g_build_path (G_DIR_SEPARATOR_S, prefix, "ontologies", "20-dc", NULL);
 	test_schemas[1] = g_build_path (G_DIR_SEPARATOR_S, prefix, "ontologies", "31-nao", NULL);
-	test_schemas[2] = g_build_path (G_DIR_SEPARATOR_S, build_prefix, "change", "ontologies", "99-example", NULL);
+	test_schemas[2] = g_build_path (G_DIR_SEPARATOR_S, prefix, "ontologies", "90-tracker", NULL);
+	test_schemas[3] = g_build_path (G_DIR_SEPARATOR_S, build_prefix, "change", "ontologies", "99-example", NULL);
 
 	ontology_file = g_build_path (G_DIR_SEPARATOR_S, build_prefix, "change", "ontologies", "99-example.ontology", NULL);
 
@@ -264,7 +265,7 @@ test_ontology_change (void)
                 g_chmod (ontology_file, 0666);
 
 		tracker_data_manager_init (0, (const gchar **) test_schemas,
-		                           NULL, FALSE);
+		                           NULL, FALSE, NULL, NULL, NULL);
 
 		if (g_file_get_contents (update, &queries, NULL, NULL)) {
 			gchar *query = strtok (queries, "\n");
@@ -290,7 +291,7 @@ test_ontology_change (void)
 	delete_db (FALSE);
 
 	tracker_data_manager_init (0, (const gchar **) test_schemas,
-	                           NULL, TRUE);
+	                           NULL, TRUE, NULL, NULL, NULL);
 
 	for (i = 0; change_tests[i].test_name != NULL; i++) {
 		gchar *query_filename;
@@ -327,7 +328,10 @@ test_ontology_init (void)
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           NULL,
 	                           NULL,
-	                           FALSE);
+	                           FALSE,
+	                           NULL,
+	                           NULL,
+	                           NULL);
 
 	tracker_data_manager_shutdown ();
 
@@ -335,7 +339,10 @@ test_ontology_init (void)
 	tracker_data_manager_init (0,
 	                           NULL,
 	                           NULL,
-	                           FALSE);
+	                           FALSE,
+	                           NULL,
+	                           NULL,
+	                           NULL);
 
 	tracker_data_manager_shutdown ();
 }
@@ -363,7 +370,10 @@ test_query (gconstpointer test_data)
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           NULL,
 	                           NULL, 
-	                           FALSE);
+	                           FALSE,
+	                           NULL,
+	                           NULL,
+	                           NULL);
 
 	/* load data set */
 	data_filename = g_strconcat (data_prefix, ".ttl", NULL);
